/**
 * Copyright 2008,2009 Ivan SZKIBA
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * under the License.
 */
package net.sourceforge.tink.app.cli;

import net.sourceforge.tink.app.TinkAppHelper;
import net.sourceforge.tink.app.engine.Engine;
import net.sourceforge.tink.app.engine.SiteConfig;
import net.sourceforge.tink.model.Output;
import net.sourceforge.tink.model.TinkException;
import net.sourceforge.tink.model.helpers.ConsoleOutput;
import net.sourceforge.tink.model.helpers.ConsoleOutput.Verbosity;

import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.OptionGroup;
import org.apache.commons.cli.Options;

import org.ini4j.Ini;

import java.util.logging.Level;
import java.util.logging.Logger;

public class TinkCmd extends TinkAppHelper
{
    public static final String PROGRAM_NAME = "tink";
    private static final Logger logger = Logger.getLogger(TinkCmd.class.getName());

    public static enum Action
    {
        PROCESS
        {
            @Override public void execute(Engine engine, String siteName, SiteConfig siteConfig) throws TinkException
            {
                engine.doProcess(siteName, siteConfig);
            }
        },
        PUBLISH
        {
            @Override public void execute(Engine engine, String siteName, SiteConfig siteConfig) throws TinkException
            {
                engine.doPublish(siteName, siteConfig);
            }
        },
        EASY
        {
            @Override public void execute(Engine engine, String siteName, SiteConfig siteConfig) throws TinkException
            {
                engine.doProcessAndPublish(siteName, siteConfig);
            }
        },
        UPDATE
        {
            @Override public void execute(Engine engine, String siteName, SiteConfig siteConfig) throws TinkException
            {
                engine.doUpdateConfig(siteName, siteConfig);
            }
        };
        public abstract void execute(Engine engine, String siteName, SiteConfig siteConfig) throws TinkException;

        public static Action fromString(String str)
        {
            return valueOf(str.toUpperCase());
        }
    }

    private Action action;
    private Engine engine;
    private Option optAction;
    private Option optDebug;
    private OptionGroup optGroupOutput;
    private Option optOutput;
    private Option optProperty;
    private Option optQuiet;
    private Option optSite;
    private Option optVerbose;
    private Output output;
    private SiteConfig siteConfig;
    private String siteName;

    public TinkCmd()
    {
        super(PROGRAM_NAME);
    }

    public static void main(String[] args)
    {
        TinkCmd instance = new TinkCmd();

        try
        {
            instance.process(args);
        }
        catch (TinkException x)
        {
            logger.log(Level.SEVERE, x.getMessage(), x);
            System.exit(1);
        }

        System.exit(0);
    }

    protected Action buildAction()
    {
        String str = getOption(optAction);

        return (str == null) ? Action.EASY : Action.fromString(str);
    }

    protected Engine buildEngine()
    {
        return new Engine(output, getConfigFile());
    }

    @SuppressWarnings("static-access")
    @Override protected Options buildOptions()
    {
        Options opts = super.buildOptions();

        optAction = OptionBuilder.withDescription("Action (process,publish,easy,update").withLongOpt("action").hasArg().withArgName("name").create('a');
        optOutput = OptionBuilder.withDescription("Output verbosity (debug,verbose,info,warning,error)").withLongOpt("output").hasArg().withArgName("value").create('o');
        optSite = OptionBuilder.withDescription("Site name").withLongOpt("site").hasArg().withArgName("name").create('s');
        optVerbose = OptionBuilder.withDescription("Be verbose").withLongOpt("verbose").create('v');
        optQuiet = OptionBuilder.withDescription("Be quiet").withLongOpt("quiet").create('q');
        optDebug = OptionBuilder.withDescription("Debug output").withLongOpt("debug").create('d');
        optProperty = OptionBuilder.withDescription("Set site config property").withLongOpt("property").hasArgs().withValueSeparator().withArgName("value").create('p');
        optGroupOutput = new OptionGroup();

        optGroupOutput.addOption(optVerbose);
        optGroupOutput.addOption(optQuiet);
        optGroupOutput.addOption(optDebug);
        optGroupOutput.addOption(optOutput);
        opts.addOptionGroup(optGroupOutput);
        opts.addOption(optAction);
        opts.addOption(optSite);
        opts.addOption(optProperty);

        return opts;
    }

    protected Output buildOutput()
    {
        Verbosity verbosity = Verbosity.INFO;

        if (hasOption(optOutput))
        {
            verbosity = Verbosity.fromString(optOutput.getValue());
        }
        else if (hasOption(optDebug))
        {
            verbosity = Verbosity.DEBUG;
        }
        else if (hasOption(optVerbose))
        {
            verbosity = Verbosity.VERBOSE;
        }
        else if (hasOption(optQuiet))
        {
            verbosity = Verbosity.WARNING;
        }

        return new ConsoleOutput(verbosity);
    }

    protected SiteConfig buildSiteConfig() throws TinkException
    {
        Ini init = new Ini();
        Ini.Section section = init.add(siteName);

        if (hasOption(optProperty))
        {
            String[] values = optProperty.getValues();

            for (int i = 0; i < values.length; i += 2)
            {
                section.put(values[i], values[i + 1]);
            }
        }

        return section.to(SiteConfig.class);
    }

    protected String buildSiteName()
    {
        return getOption(optSite);
    }

    protected void process(String[] args) throws TinkException
    {
        if (init(args))
        {
            action.execute(engine, siteName, siteConfig);
        }
    }

    @Override protected void processOptions() throws TinkException
    {
        super.processOptions();
        output = buildOutput();
        siteName = buildSiteName();
        siteConfig = buildSiteConfig();
        engine = buildEngine();
        action = buildAction();
        if (!getConfigFile().exists() && !action.equals(Action.UPDATE))
        {
            throw new TinkException("Config file doesn't exists: " + getConfigFile().getAbsolutePath());
        }
    }
}
