/**
 * Copyright 2008,2009 Ivan SZKIBA
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * under the License.
 */
package net.sourceforge.tink.app.engine;

import net.sourceforge.tink.model.FileObject;
import net.sourceforge.tink.model.Output;
import net.sourceforge.tink.model.TinkException;
import net.sourceforge.tink.model.helpers.ProcessorBuilder;
import net.sourceforge.tink.model.helpers.PublisherBuilder;

import org.ini4j.Ini;

import java.beans.Introspector;
import java.beans.PropertyDescriptor;

import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import java.lang.reflect.Method;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Engine
{
    public static final String CONFIG_SECTION = "engine";
    private static final Logger logger = Logger.getLogger(Engine.class.getName());
    private EngineConfig config;
    private FileObject configFile;
    private Ini ini;
    private final Output output;

    public Engine(Output output, FileObject configFile)
    {
        this.output = output;
        this.configFile = configFile;
    }

    public synchronized FileObject getConfigFile()
    {
        return configFile;
    }

    public synchronized void setConfigFile(FileObject value)
    {
        configFile = value;
        ini = null;
        config = null;
    }

    public void doProcess(String siteName, SiteConfig siteConfig) throws TinkException
    {
        output.info("Processing site %s", siteName);
        doProcess(newSiteConfigWrapper(siteName, siteConfig));
    }

    public void doProcessAndPublish(String siteName, SiteConfig siteConfig) throws TinkException
    {
        output.info("Processing and publishing site %s", siteName);
        SiteConfigWrapper common = newSiteConfigWrapper(siteName, siteConfig);

        doProcess(common);
        doPublish(common);
    }

    public void doPublish(String siteName, SiteConfig siteConfig) throws TinkException
    {
        output.info("Publishing site %s", siteName);
        doPublish(newSiteConfigWrapper(siteName, siteConfig));
    }

    public void doUpdateConfig(String siteName, SiteConfig siteConfig) throws TinkException
    {
        output.info("Updating configuration for site %s", siteName);
        getSection(siteName).putAll(bean2map(siteConfig));
        saveConfig();
    }

    protected static Map<String, String> bean2map(Object bean)
    {
        Map<String, String> map = new HashMap<String, String>();

        try
        {
            PropertyDescriptor[] properties = Introspector.getBeanInfo(bean.getClass()).getPropertyDescriptors();

            for (PropertyDescriptor prop : properties)
            {
                Method method = prop.getReadMethod();

                if (method != null)
                {
                    Object value = method.invoke(bean, (Object[]) null);

                    if (value != null)
                    {
                        map.put(prop.getName(), value.toString());
                    }
                }
            }
        }
        catch (Exception x)
        {
            logger.log(Level.SEVERE, null, x);
        }

        return map;
    }

    protected synchronized Ini getIni() throws TinkException
    {
        if (ini == null)
        {
            output.verbose("Loading config file %s", configFile.getAbsolutePath());
            try
            {
                ini = new Ini(configFile.openReader());
                config = getSection(CONFIG_SECTION).to(EngineConfig.class);
            }
            catch (IOException x)
            {
                throw new TinkException(x);
            }
        }

        return ini;
    }

    protected synchronized Ini.Section getSection(String name) throws TinkException
    {
        Ini.Section section = getIni().get(name);

        if (section == null)
        {
            output.verbose("Missing %s config section, creating new", name);
            section = ini.add(name);
        }

        return section;
    }

    protected SiteConfig getSiteConfig(String name) throws TinkException
    {
        return getSection(name).to(SiteConfig.class);
    }

    protected void doProcess(SiteConfigWrapper config) throws TinkException
    {
        try
        {
            new ProcessorBuilder(config).buildProcessor().execute();
        }
        finally
        {
            config.cleanup();
        }
    }

    protected void doPublish(SiteConfigWrapper config) throws TinkException
    {
        try
        {
            new PublisherBuilder(config).buildPublisher().execute();
        }
        finally
        {
            config.cleanup();
        }
    }

    protected SiteConfigWrapper newSiteConfigWrapper(String siteName, SiteConfig siteConfig) throws TinkException
    {
        return new SiteConfigWrapper(siteConfig, getSiteConfig(siteName), output);
    }

    protected void saveConfig() throws TinkException
    {
        try
        {
            Writer writer = new FileWriter(configFile);

            getIni().store(writer);
            writer.close();
        }
        catch (IOException x)
        {
            throw new TinkException(x);
        }
    }
}
